from collections import defaultdict

from django.core.cache import cache

from django.core.exceptions import ObjectDoesNotExist

from nltk import PorterStemmer

from sklearn.feature_extraction.text import TfidfVectorizer

from course_inspector.books.book_analyzer import calculate_book_similarity
from course_inspector.books.models import Book
from course_inspector.courses.models import Course, CurriculumObject

import pickle


# Returns course content in a stemmer-friendly way
def get_meta_terms(course):
    if course.meta_course_content:
        return course.meta_course_content.lower().replace(',', '') \
            .replace('.', '').split(' ')
    else:
        return ''


def get_or_set_cache(university):
    if cache.get('tfidf_vector') and cache.get('all_courses_list'):
        print('Cached data exists')
        # Load cached data
        tfidf_vector = pickle.loads(cache.get('tfidf_vector'))
        all_course_list = pickle.loads(cache.get('all_courses_list'))
        return tfidf_vector, all_course_list
    else:
        print('There is no cached data')
        course_filter = {} if not university else {'university': university}
        all_courses = Course.objects.filter(**course_filter)
        all_courses_content = []  # Comparison basis
        all_courses_list = []  # We need this to find the index of a course
        for c in all_courses:
            all_courses_content.append(c.meta_comparison_basis())
            all_courses_list.append(c)

        vect = TfidfVectorizer(
            stop_words='english',
            analyzer='word',
            # max_features=20,
        )
        tfidf_vector = vect.fit_transform(all_courses_content)

        cache.set('tfidf_vector', pickle.dumps(tfidf_vector))
        cache.set('all_courses_list', pickle.dumps(all_courses_list))
        return tfidf_vector, all_courses_list


def find_similar(base_course, university=None, n=10):
    result_array = []

    cache_result = get_or_set_cache(university)
    tfidf_vector = cache_result[0]
    course_list = cache_result[1]

    # Find index of base_course (to know which row to look at)
    base_course_index = 0
    for index, item in enumerate(course_list):
        if item == base_course:
            base_course_index = index

    # Do vector space model calculations, and get the row where base_course is
    # located
    result_vector = (tfidf_vector * tfidf_vector.T).A[base_course_index]

    # Finds the indexes of the top n+1 results, and returns them in sorted
    # order
    top_results_indexes = sorted(range(len(result_vector)),
                                 key=lambda i: result_vector[i])[-n+1:]

    for i in top_results_indexes:
        course = course_list[i]
        result_array.append({
            'university': course.university.name,
            'university_short_name': course.university.short_name,
            'name': course.name,
            'id': course.id,
            'code': course.course_id,
            'score': round(result_vector[i], 4),
        })

    # Reverse the list
    result_array = list(reversed(result_array))

    # Pop the top result (which is just the course compared to itself)
    result_array = result_array[1:]

    return {
        'res': result_array,
    }


def compare_courses(course1, course2):
    result_dict = {}
    curriculum_similarity_scores = []

    # Curriculum similarity
    try:
        result_dict['curriculum_similarity'] = {}
        result_dict['curriculum_similarity']['detailed'] = {}

        curriculum_course1 = CurriculumObject.objects.filter(
            course=course1.id).values_list('book', 'excluded_chapters')
        curriculum_course2 = CurriculumObject.objects.filter(
            course=course2.id).values_list('book', 'excluded_chapters')

        # Organize curriculum for easier
        course1_curriculum_sorted = defaultdict(list)
        for curr_element in curriculum_course1:
            course1_curriculum_sorted[curr_element[0]].append(curr_element[1])

        course2_curriculum_sorted = defaultdict(list)
        for curr_element in curriculum_course2:
            course2_curriculum_sorted[curr_element[0]].append(curr_element[1])

        # Compare every book vs book set
        for curriculum1 in course1_curriculum_sorted.items():
            for curriculum2 in course2_curriculum_sorted.items():
                excluded_chapters = {
                    'book1_excluded': curriculum1[1],
                    'book2_excluded': curriculum2[1],
                }

                book1 = Book.objects.get(isbn=curriculum1[0])
                book2 = Book.objects.get(isbn=curriculum2[0])
                similarity = calculate_book_similarity(
                    book1,
                    book2,
                    excluded_chapters,
                )
                curriculum_similarity_scores.append(similarity.get('res'))
                result_dict['curriculum_similarity']['detailed'][
                    book1.title + ' - ' + book2.title] = similarity.get('res')

        if curriculum_similarity_scores:
            result_dict['curriculum_similarity']['overall'] = sum(
                curriculum_similarity_scores) / len(
                curriculum_similarity_scores)

    except ObjectDoesNotExist:
        result_dict['curriculum_similarity'] = 'Course(s) missing curriculum'

    # Overall similarity (description + goals)
    vect = TfidfVectorizer(
        stop_words='english',
        analyzer='word',
        max_features=10,
    )

    # Learn vocabulary and idf, return term-document matrix.
    tfidf = vect.fit_transform([
        course1.meta_comparison_basis(),
        course2.meta_comparison_basis(),
    ])

    print(tfidf)

    result_dict['overall_similarity'] = (tfidf * tfidf.T).A[0][1]

    stemmer = PorterStemmer()
    stemmed1 = ' '.join([str(stemmer.stem(token)) for token in
                         get_meta_terms(course1)])
    stemmed2 = ' '.join([str(stemmer.stem(token)) for token in
                         get_meta_terms(course2)])

    # Description similarity
    if course1.meta_course_content and course2.meta_course_content:
        tfidf = vect.fit_transform([
            # course1.meta_course_content, course2.meta_course_content,
            stemmed1, stemmed2
        ])
        result_dict['description_similarity'] = (tfidf * tfidf.T).A[0][1]
        # print(dict(zip(vect.get_feature_names(), vect.idf_)))
        # print(vect.stop_words_)
        # print(vect.get_feature_names())

    return result_dict
