# -*- coding: utf-8 -*-
import re
from django.core.management import BaseCommand
from django.db import IntegrityError
import requests

from course_inspector.courses.models import Course, University


black_listed_courses = [
    'TPK4520', 'TPK4420', 'TPD4500', 'TPD4505', 'TMR4243',
    'TMR4240', 'AAR4528', 'AAR4863', 'TPK4530', 'TPK4430',
    'AAR4611', 'AAR4909', 'AAR4623', 'AAR4904', 'AAR4602',
    'AAR4903', 'AAR4814', 'AAR4510', 'AAR4851', 'AAR4540',
    'rIMT1241', 'IMT2261', 'GEO2311', 'IMT1441', 'AAR4552',
    'AAR4881', 'TPK4440', 'TPK4540', 'AAR4890', 'AAR4570',
    'LAT1001', 'AAR4565', 'AAR4885', 'LAT1004', 'KLAS2001',
    'KLAS3000', 'KLAS2000', 'TEK2031', 'MAL4000']

black_listed_study_programmes = [
    'AAR', 'KLAS',
]


class Command(BaseCommand):
    help = 'Scrapes the IME API for courses'

    def handle(self, *args, **options):
        r = requests.get('http://www.ime.ntnu.no/api/course/en/-')
        data = r.json()

        for course in data['course']:
            # Handle this course
            process_course(course['code'])
            if Course.objects.filter(course_id=course['code']):
                continue


def process_course(code):
    try:
        return Course.objects.get(course_id=code)
    except Course.DoesNotExist:
        pass

    if code in black_listed_courses:
        return None
    if any(blacklist_course in code for blacklist_course in black_listed_study_programmes):
        return None

    ntnu, created = University.objects.get_or_create(
        short_name='ntnu',
        name="NTNU",
    )

    print("Working on " + code)
    r = requests.get('http://www.ime.ntnu.no/api/course/en/' + code)
    data = r.json()['course']

    if not data:
        print("No course data found for {}".format(code))
        return None

    new_course = Course()

    if 'code' not in data:
        print("Course missing code")
        return None
    new_course.course_id = data['code']

    if 'name' in data:
        new_course.name = data['name']
    else:
        print("Course missing name")

    if 'credit' in data:
        new_course.credits = data['credit']
    else:
        print("Course missing credit")

    if 'location' in data:
        new_course.meta_location = data['location']
    else:
        print("Course missing location")

    if 'subjectArea' in data:
        new_course.meta_subject_area = data['subjectArea']
    else:
        print("Course missing subjectArea")

    if 'usedInStudyprogrammes' in data:
        new_course.meta_used_in_studyprogrammes = data['usedInStudyprogrammes']
    else:
        print("Course missing usedInStudyprogrammes")

    if 'studyLevelName' in data:
        new_course.meta_study_level_name = data['studyLevelName']
    else:
        print("Course missing studyLevelName")

    new_course.study_level_code = data.get('studyLevelCode')
    new_course.semester = data.get('taughtFromTerm', '').lower()
    new_course.university = ntnu

    infotype = None

    if 'infoType' in data:
        infotype = data['infoType']
    else:
        print('This course has no infoType')

    if infotype:
        for info in infotype:
            if info['code'] == 'ANBFORK' or info['code'] == 'FORK':
                fork_type = info['code']
                if fork_type == 'ANBFORK':
                    fork_type = 'recommended'
                else:
                    fork_type = 'required'

                fork_plain_text = ''

                # Get ANBFORK from norwegian endpoint as well
                if info['code'] == 'ANBFORK':
                    r = requests.get('http://www.ime.ntnu.no/api/course/' + code)
                    nor_data = r.json()['course'].get('infoType', [])
                    for nor_infotype in nor_data:
                        if nor_infotype.get('code') == 'ANBFORK':
                            fork_plain_text = fork_plain_text + nor_infotype.get('text', '')

                if 'text' in info:
                    fork_plain_text = fork_plain_text + info['text']
                    dependency_courses = re.findall(
                        '[a-zA-z]{2,4}\d{2,6}',
                        fork_plain_text,
                        # ([a-zA-z]{2,4}\d{2,6}(/\d{2})*)|([a-zA-z]{2,4}\d{2,6})
                    )

                    if fork_type == 'recommended':
                        new_course.meta_recommended_prereq_knowledge = \
                            info['text']
                    else:
                        new_course.meta_prereq_knowledge = info['text']

                    for new_child in dependency_courses:
                        if new_child == new_course.course_id:
                            continue

                        # Sjekk om new_child er blacklisted før vi går videre.
                        # Skriv til loggfil
                        if new_child in black_listed_courses:
                            text_file = open("output.txt", "a")
                            text_file.write("Encountered blacklisted course {}\n".format(new_child))
                            text_file.close()
                            continue
                        if any(study_code in new_child for study_code in black_listed_study_programmes):
                            text_file = open("output.txt", "a")
                            text_file.write("Encountered blacklisted study programme {}\n".format(new_child))
                            text_file.close()
                            continue

                        child = process_course(new_child)
                        if child:
                            try:
                                new_course.add_child(child, type=fork_type)
                            except IntegrityError:
                                print("Child {} already registered".format(
                                    child.course_id))
                else:
                    print('This course has no ANBFORK or FORK')

            if info['code'] == 'E-URL':
                if 'text' in info:
                    new_course.meta_e_url = info['text']
                else:
                    print('This course has no E-URL')

            if info['code'] == 'INNHOLD':
                if 'text' in info:
                    new_course.meta_course_content = info['text']
                else:
                    print('This course has no INNHOLD')

            if info['code'] == 'KURSMAT':
                if 'text' in info:
                    new_course.meta_course_material = info['text']
                else:
                    print('This course has no KURSMAT')

            if info['code'] == 'LÆRFORM':
                if 'text' in info:
                    new_course.meta_learning_methods = info['text']
                else:
                    print('This course has no LÆRFORM')

            if info['code'] == 'MÅL':
                if 'text' in info:
                    new_course.meta_goals = info['text']
                else:
                    print('This course has no MÅL')

    print('\t\t\t\tFinished processing {}\n\n\n'.format(
        new_course.course_id))
    try:
        new_course.save()
    except IntegrityError:
        print('Course already exists')

    return new_course
