import uuid

from django.db import models
from django.db.models import Q
from django.utils import timezone

from django_dag.models import edge_factory, node_factory

from course_inspector.books.models import Book, Chapter


class University(models.Model):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    name = models.CharField(max_length=50)
    short_name = models.CharField(max_length=10, unique=True)

    class Meta:
        ordering = ['name']

    def __str__(self):
        return self.name


class Course(node_factory('CourseRelation')):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    created_at = models.DateTimeField(blank=True, null=True)
    course_id = models.CharField(max_length=32, null=True, blank=True)
    credits = models.DecimalField(
        max_digits=5, decimal_places=3, blank=True, null=True)
    name = models.CharField(max_length=900, blank=True)
    university = models.ForeignKey(University, on_delete=models.CASCADE)

    AUTUMN = 'autumn'
    SPRING = 'spring'
    SEMESTER_CHOICES = (
        (AUTUMN, 'Autumn'),
        (SPRING, 'Spring'),
    )

    semester = models.CharField(
        max_length=32, blank=True, null=True, choices=SEMESTER_CHOICES)

    STUDY_LEVEL_CHOICES = (
        ('', 'Unknown'),
        ('LN', 'Videregående emner, nivå II'),
        ('', ''),
    )
    study_level = models.CharField(
        choices=STUDY_LEVEL_CHOICES, blank=True, max_length=1500)
    study_level_code = models.IntegerField(blank=True, null=True)

    meta_goals = models.TextField(max_length=50000, blank=True)
    meta_location = models.CharField(max_length=9000, null=True, blank=True)
    meta_e_url = models.URLField(max_length=500, null=True, blank=True)
    meta_course_content = models.TextField(
        max_length=50000,
        null=True,
        blank=True,
    )
    meta_course_material = models.TextField(
        max_length=50000,
        null=True,
        blank=True,
    )
    meta_recommended_prereq_knowledge = models.TextField(
        max_length=50000,
        null=True,
        blank=True,
    )
    meta_prereq_knowledge = models.TextField(
        max_length=50000,
        null=True,
        blank=True,
    )
    meta_subject_area = models.CharField(
        max_length=50000,
        null=True,
        blank=True,
    )
    meta_used_in_studyprogrammes = models.CharField(
        max_length=50000,
        null=True,
        blank=True,
    )
    meta_learning_methods = models.TextField(
        max_length=50000,
        null=True,
        blank=True,

    )
    meta_study_level_name = models.CharField(
        max_length=900,
        null=True,
        blank=True,
    )

    external_id = models.CharField(max_length=10, null=True, blank=True)

    class Meta:
        ordering = ['course_id']
        unique_together = (('university', 'course_id'),)

    def __str__(self):
        return '{}_{}'.format(self.university.short_name, self.course_id)

    def meta_comparison_basis(self):
        if not self.meta_course_content and not self.meta_goals:
            return self.name
        elif not self.meta_course_content and self.meta_goals:
            return self.meta_goals
        elif self.meta_course_content and not self.meta_goals:
            return self.meta_course_content
        else:
            return self.meta_course_content + ' ' + self.meta_goals

    def save(self, *args, **kwargs):
        if self.created_at is None:
            self.created_at = timezone.now()
        super().save(*args, **kwargs)

    def get_related(self):
        return ApprovedCourseRelation.find_all(self)


class CourseRelation(edge_factory(Course, concrete=False)):
    EDGE_TYPES = (
        ('required', 'Required'),
        ('recommended', 'Recommended'),
    )
    type = models.CharField(choices=EDGE_TYPES, max_length=900, blank=True)

    class Meta:
        unique_together = (('parent', 'child'),)

    def __str__(self):
        return '{} parent of {}'.format(self.parent, self.child)


class CurriculumObject(models.Model):
    book = models.ForeignKey(Book)
    course = models.ForeignKey(Course)
    excluded_chapters = models.ManyToManyField(Chapter, blank=True)

    @property
    def isbn(self):
        return self.book.isbn

    @property
    def publisher(self):
        return self.book.publisher.name

    def __str__(self):
        return '{} in course {}'.format(self.book.title, self.course.course_id)


class ApprovedCourseRelation(models.Model):
    created_at = models.DateTimeField(blank=True, null=True)
    # Just a string with the name of the one that approved the course
    created_by = models.CharField(max_length=512)
    course_a = models.ForeignKey(Course, related_name='course_a')
    course_b = models.ForeignKey(Course, related_name='course_b')

    def __str__(self):
        return 'Relation between {} and {}'.format(
            self.course_a, self.course_b)

    def save(self, *args, **kwargs):
        if self.created_at is None:
            self.created_at = timezone.now()
        super().save(*args, **kwargs)

    @classmethod
    def find_all(cls, course):
        return cls.objects.filter(Q(course_a=course) | Q(course_b=course))
