import zlib

from django.conf import settings

from course_inspector.courses.models import CourseRelation


class PlantUML:
    def __init__(self, root_course):
        self.traversed_courses = []
        self.root_course = root_course

    def generate_plantuml_markup(self):
        uml_tree = '@startuml\n'
        uml_tree += self._get_markup_for_course(self.root_course)
        uml_tree += '@enduml'
        return uml_tree

    def get_plant_uml_img(self, plant_uml_markup):
        # print(plant_uml_markup)
        return self._get_url(plant_uml_markup)

    def _get_markup_for_course(self, course, parent=None):
        if course in self.traversed_courses:
            return self._get_plant_relation(parent, course)
        self.traversed_courses.append(course)

        uml_tree = self._get_plant_class(course)
        if parent:
            uml_tree += self._get_plant_relation(parent, course)

        for child in course.children.all():
            uml_tree += self._get_markup_for_course(child, parent=course)
        return uml_tree

    def _get_url(self, text):
        url = 'https://www.plantuml.com/plantuml/svg/'
        return url + self._deflate_and_encode(text)

    def _get_plant_class(self, course):
        template = 'class {} {} {{\n{}  \n}}\n'
        url = settings.COURSEWEB_URL
        course_url = '[[{}/course/{}/{}]]'.format(url,
                                                  course.university.short_name,
                                                  course.course_id)
        return template.format(course.course_id, course_url, course.name)

    def _get_plant_relation(self, parent, child):
        relation = CourseRelation.objects.get(parent=parent, child=child)

        # Find and ignore transitive relations
        # Relations where child is a child
        childs_child_relations = CourseRelation.objects.filter(child=child)
        for rel in childs_child_relations:
            parents_child_relations = CourseRelation.objects.filter(
                child=rel.parent
            )
            for parent_relation in parents_child_relations:
                if parent_relation.parent == parent:
                    # We found a transitive relation. Do not draw it
                    return ''

        if relation.type == 'required':
            return '{} --> {} : Required \n'.format(
                child.course_id, parent.course_id)
        return '{} --> {}\n'.format(child.course_id, parent.course_id)

    def _deflate_and_encode(self, plantuml_text):
        zlibbed_str = zlib.compress(plantuml_text.encode())
        compressed_string = zlibbed_str[2:-4]
        return self._encode(compressed_string)

    def _encode(self, data):
        res = ""
        for i in range(0, len(data), 3):
            if i+2 == len(data):
                res += self._encode3bytes(
                    ord(chr(data[i])), ord(chr(data[i+1])), 0)
            elif i+1 == len(data):
                res += self._encode3bytes(ord(chr(data[i])), 0, 0)
            else:
                res += self._encode3bytes(
                    ord(chr(data[i])),
                    ord(chr(data[i+1])),
                    ord(chr(data[i+2]))
                )
        return res

    def _encode3bytes(self, b1, b2, b3):
        c1 = b1 >> 2
        c2 = ((b1 & 0x3) << 4) | (b2 >> 4)
        c3 = ((b2 & 0xF) << 2) | (b3 >> 6)
        c4 = b3 & 0x3F
        res = ""
        res += self._encode6bit(c1 & 0x3F)
        res += self._encode6bit(c2 & 0x3F)
        res += self._encode6bit(c3 & 0x3F)
        res += self._encode6bit(c4 & 0x3F)
        return res

    def _encode6bit(self, b):
        if b < 10:
            return chr(48 + b)
        b -= 10
        if b < 26:
            return chr(65 + b)
        b -= 26
        if b < 26:
            return chr(97 + b)
        b -= 26
        if b == 0:
            return '-'
        if b == 1:
            return '_'
        return '?'
