from rest_framework import serializers

from course_inspector.books.models import Book
from course_inspector.courses.models import (
    ApprovedCourseRelation,
    Course,
    CourseRelation,
    CurriculumObject,
    University,
)

from course_inspector.courses.plant_uml_tree import PlantUML


class UniversitySerializer(serializers.ModelSerializer):
    class Meta:
        model = University
        fields = (
            'id',
            'name',
            'short_name',
        )


class BookSerializer(serializers.ModelSerializer):
    class Meta:
        model = Book
        fields = (
            'title',
        )


class CourseChildSerializer(serializers.ModelSerializer):
    required = serializers.SerializerMethodField()
    university = UniversitySerializer()

    class Meta:
        model = Course
        fields = (
            'id',
            'course_id',
            'name',
            'required',
            'meta_study_level_name',
            'credits',
            'university'
        )

    def get_fields(self):
        # Shitty hack to remove required field if this serializer
        # is called from ApprovedCourseListSerializer
        fields = super(self.__class__, self).get_fields()
        if not self._kwargs.get('context'):
            fields.pop('required')
        return fields

    def get_required(self, course):
        parent = self._kwargs.get('context')
        relation = CourseRelation.objects.get(parent=parent, child=course)
        return relation.type == 'required'


class CurriculumObjectSerializer(serializers.ModelSerializer):
    book = BookSerializer()

    class Meta:
        model = CurriculumObject
        fields = (
            'book',
            'excluded_chapters',
            'isbn',
            'publisher',
        )


class ApprovedCourseListSerializer(serializers.ModelSerializer):
    course = serializers.SerializerMethodField()

    class Meta:
        model = ApprovedCourseRelation
        fields = (
            'created_at',
            'created_by',
            'course',
        )

    def get_course(self, rel):
        parent = self._kwargs.get('context')
        course = rel.course_a if rel.course_a != parent else rel.course_b
        return CourseChildSerializer(course).data


class ApprovedCourseRelationSerializer(serializers.ModelSerializer):
    course_a = serializers.PrimaryKeyRelatedField(
        queryset=Course.objects.all())
    course_b = serializers.PrimaryKeyRelatedField(
        queryset=Course.objects.all())

    class Meta:
        model = ApprovedCourseRelation


class CourseSerializer(serializers.ModelSerializer):
    university = UniversitySerializer()
    children = serializers.SerializerMethodField()
    approved_courses = serializers.SerializerMethodField()
    plantuml_url = serializers.SerializerMethodField()
    curriculum = CurriculumObjectSerializer(
        source='curriculumobject_set', many=True, required=False)

    class Meta:
        model = Course
        fields = (
            'id',
            'course_id',
            'university',
            'children',
            'approved_courses',
            'name',
            'university',
            'created_at',
            'semester',
            'credits',
            'plantuml_url',
            'curriculum',
            'meta_e_url',
            'meta_goals',
            'meta_learning_methods',
            'meta_course_content',
            'meta_study_level_name',
        )

    def get_fields(self):
        fields = super(self.__class__, self).get_fields()
        if self.context['view'].action in ('create', 'update'):
            fields['university'] = serializers.PrimaryKeyRelatedField(
                queryset=University.objects.all())
        return fields

    def get_plantuml_url(self, course):
        uml_graph = PlantUML(course)
        uml_markup = uml_graph.generate_plantuml_markup()
        return uml_graph.get_plant_uml_img(uml_markup)

    def get_children(self, course):
        children = course.children.all()
        return CourseChildSerializer(children, many=True, context=course).data

    def get_approved_courses(self, course):
        approved = course.get_related()
        return ApprovedCourseListSerializer(
            approved, many=True, context=course).data


class CourseListSerializer(serializers.ModelSerializer):
    university = UniversitySerializer()

    class Meta:
        model = Course
        fields = (
            'id',
            'course_id',
            'name',
            'university',
        )
