import uuid

from django.db.models import Q
from django.shortcuts import get_object_or_404

from rest_framework import decorators, mixins, response, viewsets
from rest_framework.permissions import IsAuthenticated

from course_inspector.api.permissions import AdminAccessPermission
from course_inspector.courses.course_comparator import (
    compare_courses,
    find_similar,
)
from course_inspector.courses.models import (
    ApprovedCourseRelation,
    Course,
    University,
)
from course_inspector.courses.serializers import (
    ApprovedCourseRelationSerializer,
    CourseListSerializer,
    CourseSerializer,
    UniversitySerializer,
)


class CourseAPIViewSet(viewsets.ModelViewSet):
    queryset = Course.objects.all().prefetch_related('curriculumobject_set')

    def get_object(self):
        queryset = self.get_queryset()
        pk = self.kwargs[self.lookup_field]
        try:
            uuid.UUID(pk)
            obj = get_object_or_404(queryset, pk=pk)
        except ValueError:
            obj = get_object_or_404(queryset, course_id=pk.upper())

        self.check_object_permissions(self.request, obj)
        return obj

    def get_serializer_class(self):
        if self.action == 'list':
            return CourseListSerializer
        else:
            return CourseSerializer

    def get_permissions(self):
        open_endpoints = (
            'list', 'retrieve', 'search', 'find_similar',
            'find_similar_from_university', 'compare',
        )
        if self.action in open_endpoints:
            self.permission_classes = []
        else:
            self.permission_classes = [IsAuthenticated, AdminAccessPermission]
        return super(CourseAPIViewSet, self).get_permissions()

    @decorators.list_route()
    def search(self, request):
        query = request.query_params.get('q')
        university = request.query_params.get('university')
        kwargs = {'university__short_name': university} if university else {}
        result = Course.objects.filter(
            Q(name__icontains=query) | Q(course_id__icontains=query),
            **kwargs,
        )[:20]

        data = {'result': CourseListSerializer(result, many=True).data}
        return response.Response(data=data)

    @decorators.detail_route(url_path='find_similar')
    def find_similar(self, request, pk=None, university_short_name=None):
        course = self.get_object()
        similar_courses = find_similar(course)
        return response.Response(data={'similar_courses': similar_courses})

    @decorators.detail_route(
        url_path='find_similar/(?P<uni_name>[A-Za-z0-9]+)'
    )
    def find_similar_from_university(self, request, *args, **kwargs):
        course = self.get_object()
        university = University.objects.get(short_name=kwargs['uni_name'])
        similar_courses = find_similar(course, university, 20)
        return response.Response(data={'similar_courses': similar_courses})

    @decorators.detail_route(
        url_path='compare/(?P<compare_course>[-A-Za-z0-9]+)'
    )
    def compare(self, request, pk=None, compare_course=None):
        course1 = self.get_object()
        course2 = Course.objects.get(id=compare_course)
        print(course1)
        print(course2)
        compared_courses = compare_courses(course1, course2)
        return response.Response(data={
            'similarity': compared_courses,
            'course1': CourseSerializer(
                course1,
                context=self.get_serializer_context()).data,
            'course2': CourseSerializer(
                course2,
                context=self.get_serializer_context()).data,

        })

    @decorators.detail_route(
        url_path='add_prerequisite/(?P<prereq_pk>[-A-Za-z0-9]+)'
    )
    def add_prerequisite(self, request, pk=None, prereq_pk=None):
        course = self.get_object()
        new_prereq_course = get_object_or_404(Course, pk=uuid.UUID(prereq_pk))
        course.add_child(new_prereq_course)
        course.save()
        return response.Response(data='Course added')

    @decorators.detail_route(
        url_path='remove_prerequisite/(?P<prereq_pk>[-A-Za-z0-9]+)'
    )
    def remove_prerequisite(self, request, pk=None, prereq_pk=None):
        course = self.get_object()
        pre_req = get_object_or_404(Course, pk=uuid.UUID(prereq_pk))
        course.remove_child(pre_req)
        course.save()
        return response.Response(data='Course removed')

    @decorators.detail_route(
        url_path='approve/(?P<approve_pk>[-A-Za-z0-9]+)'
    )
    def add_approved(self, request, pk=None, approve_pk=None):
        course = self.get_object()
        to_approve = get_object_or_404(Course, pk=uuid.UUID(approve_pk))
        ApprovedCourseRelation.objects.create(
            course_a=course, course_b=to_approve
        )
        return response.Response(data='Course added')

    @decorators.detail_route(
        url_path='remove_approved/(?P<approve_pk>[-A-Za-z0-9]+)'
    )
    def remove_approved(self, request, pk=None, approve_pk=None):
        course = self.get_object()
        approved = get_object_or_404(Course, pk=uuid.UUID(approve_pk))
        relations = ApprovedCourseRelation.objects.filter(
            Q(course_a=course, course_b=approved) |
            Q(course_a=approved, course_b=course)
        )
        relations.delete()

        return response.Response(data='Course removed')


class CourseUniversityAPIViewSet(CourseAPIViewSet):
    def get_queryset(self):
        return super().get_queryset().filter(
            university__short_name=self.kwargs['university_short_name']
        )

    def get_object(self):
        pk = self.kwargs[self.lookup_field]
        queryset = self.get_queryset()
        try:
            uuid.UUID(pk)
            obj = get_object_or_404(queryset, pk=pk)
        except ValueError:
            obj = get_object_or_404(queryset, course_id=pk.upper())
        self.check_object_permissions(self.request, obj)
        return obj

    def get_serializer_class(self):
        if self.action == 'list':
            return CourseListSerializer
        else:
            return CourseSerializer


class UniversityAPIViewSet(viewsets.ModelViewSet):
    queryset = University.objects.all()
    serializer_class = UniversitySerializer

    def get_permissions(self):
        if self.action in ('list', 'retrieve'):
            self.permission_classes = []
        else:
            self.permission_classes = [IsAuthenticated, AdminAccessPermission]
        return super(UniversityAPIViewSet, self).get_permissions()

    def get_object(self):
        queryset = self.get_queryset()
        pk = self.kwargs[self.lookup_field]
        try:
            uuid.UUID(pk)
            obj = get_object_or_404(queryset, pk=pk)
        except ValueError:
            obj = get_object_or_404(queryset, short_name=pk)

        self.check_object_permissions(self.request, obj)
        return obj


class ApprovedCourseRelationAPIViewSet(viewsets.GenericViewSet,
                                       mixins.CreateModelMixin):
    queryset = ApprovedCourseRelation.objects.all()
    serializer_class = ApprovedCourseRelationSerializer
    permission_classes = [IsAuthenticated, AdminAccessPermission]
