import React, { Component, PropTypes } from 'react';
import { Link } from 'react-router';
import { connect } from 'react-redux';

import Row from 'react-bootstrap/lib/Row';
import Col from 'react-bootstrap/lib/Col';
import Panel from 'react-bootstrap/lib/Panel';
import Table from 'react-bootstrap/lib/Table';
import Button from 'react-bootstrap/lib/Button';
import Glyphicon from 'react-bootstrap/lib/Glyphicon';
import FormGroup from 'react-bootstrap/lib/FormGroup';
import ControlLabel from 'react-bootstrap/lib/ControlLabel';
import Alert from 'react-bootstrap/lib/Alert';

import Modal from 'components/Modal';
import CourseSearch from 'containers/CourseSearch';
import { createStructuredSelector } from 'reselect';
import { selectCourse, selectRequestingAddApprovedSuccess } from './selectors';
import { requestAddApproved, requestRemoveApproved } from './actions';
import selectAuthWrapper from '../AuthWrapper/selectors';


class ApproveTab extends Component {
  constructor() {
    super();
    this.state = {
      showForm: false,
      selectedCourse: null,
      selectionError: false,
    };
  }

  componentWillReceiveProps(next) {
    if (next.requestingAddApprovedSuccess) {
      this.setState({ showForm: false });
    }
  }

  getPanelFooter() {
    return (
      <div className="text-right">
        { this.props.user.admin &&
        <Button onClick={() => this.toggleModal(true)}>Add course</Button>
        }

      </div>
    );
  }

  getModal() {
    return (
      <Modal
        onDismiss={() => this.toggleModal(false)}
        onSubmit={(event) => this.handleAddCourse(event)}
        title="Add approval for course"
      >
        { (false) &&
        <Alert bsStyle="danger">
          <stong>Something went wrong, please try again later</stong>
        </Alert>
        }
        <FormGroup>
          <ControlLabel>Course ID</ControlLabel>
          <CourseSearch handleChange={(c) => this.handleCourseChange(c)} />
        </FormGroup>
      </Modal>
    );
  }

  handleRemoveCourse(course) {
    this.props.dispatch(requestRemoveApproved(this.props.course.id, course));
  }

  handleAddCourse(event) {
    event.preventDefault();
    const course = this.state.selectedCourse;

    if (!course) {
      return this.setState({ selectionError: true });
    }

    return this.props.dispatch(requestAddApproved(this.props.course.id, course.id));
  }

  toggleModal(showForm) {
    this.setState({ showForm });
  }

  handleCourseChange(course) {
    this.setState({ selectedCourse: course });
  }

  render() {
    const { course } = this.props;
    const approvedCourses = course.approved_courses || [];
    return (
      <Row style={{ paddingTop: '1rem' }}>
        <Col xs={12} sm={12}>
          { this.state.showForm && this.getModal() }
          <Panel header="Approved courses" footer={this.getPanelFooter()}>
            {
              approvedCourses.length > 0 ?
                <Table responsive>
                  <thead>
                    <tr>
                      <th>Course</th>
                      <th>Name</th>
                      <th>University</th>
                      <th>Approved by</th>
                      <th>Date</th>
                      <th>Actions</th>
                    </tr>
                  </thead>
                  <tbody>
                    {approvedCourses.map((approvedCourse, key) => (
                      <tr key={key}>
                        <td>
                          <Link to={`/course/${approvedCourse.course.university.short_name}/${approvedCourse.course.course_id}`}>
                            {approvedCourse.course.course_id}
                          </Link>
                        </td>
                        <td>
                          <Link to={`/course/${approvedCourse.course.university.short_name}/${approvedCourse.course.course_id}`}>
                            {approvedCourse.course.name}
                          </Link>
                        </td>
                        <td>
                          {approvedCourse.course.university ? approvedCourse.course.university.name : ''}
                        </td>
                        <td>{approvedCourse.created_by}</td>
                        <td>{approvedCourse.created_at}</td>
                        <td>
                          <a style={{ cursor: 'pointer' }} onClick={() => this.handleRemoveCourse(approvedCourse.course.id)}>
                            <Glyphicon glyph="remove-sign" />
                          </a>
                        </td>
                      </tr>
                    ))}
                  </tbody>
                </Table>
                :
                <h5 className="text-center" style={{ color: 'grey' }}>No approved courses available</h5>
            }
          </Panel>
        </Col>
      </Row>
    );
  }
}

ApproveTab.propTypes = {
  dispatch: PropTypes.func,
  course: PropTypes.object,
  user: PropTypes.object,
};

const mapStateToProps = createStructuredSelector({
  course: selectCourse(),
  requestingAddApprovedSuccess: selectRequestingAddApprovedSuccess(),
  user: selectAuthWrapper(),
});

export default connect(mapStateToProps)(ApproveTab);
