import React, { PropTypes, Component } from 'react';
import { createStructuredSelector } from 'reselect';
import { connect } from 'react-redux';

import Row from 'react-bootstrap/lib/Row';
import Col from 'react-bootstrap/lib/Col';
import Button from 'react-bootstrap/lib/Button';
import Radio from 'react-bootstrap/lib/Radio';
import FormGroup from 'react-bootstrap/lib/FormGroup';
import FormControl from 'react-bootstrap/lib/FormControl';
import ControlLabel from 'react-bootstrap/lib/ControlLabel';
import Alert from 'react-bootstrap/lib/Alert';

import Modal from 'components/Modal';
import FieldGroup from 'components/FieldGroup';
import Spinner from 'components/Spinner';

import {
  selectUniversities,
  selectFetchingUniversities,
  selectFetchingUniversitiesError,
} from '../CreateCourse/selectors';

import {
  selectRequestingUpdateCourse,
  selectUpdateCourseSuccess,
  selectUpdateCourseFailed,
  selectCourse,
} from './selectors';
import { loadUniversityList } from '../CreateCourse/actions';
import { updateCourse } from './actions';


class UpdateCourseButton extends Component {
  constructor() {
    super();
    this.state = {
      showForm: false,
      semesterValue: null,
    };
  }

  componentWillMount() {
    if (this.props.universities.length < 1) {
      this.props.dispatch(loadUniversityList());
    }

    if (this.props.course && this.props.course.semester) {
      this.setState({ semesterValue: this.props.course.semester });
    }
  }

  componentWillReceiveProps(next) {
    if (next.updateCourseSuccess && this.state.showForm) {
      this.setState({ showForm: false });
    }
  }

  handleSemesterChange(semesterValue) {
    this.setState({ semesterValue });
  }

  handleOnClick() {
    this.setState({ showForm: true });
  }

  handleModalDismiss() {
    this.setState({ showForm: false });
  }

  handleSubmit(event) {
    event.preventDefault();

    if (this.props.requestingUpdateCourse) {
      return;
    }

    this.props.dispatch(updateCourse({
      id: this.props.course.id,
      course_id: event.currentTarget.course_id.value.trim(),
      university: event.currentTarget.university.value.trim(),
      name: event.currentTarget.name.value.trim(),
      semester: this.state.semesterValue,
      credits: event.currentTarget.credits.value.trim(),
      meta_study_level_name: event.currentTarget.level.value.trim(),
      meta_goals: event.currentTarget.goals.value.trim(),
      meta_learning_methods: event.currentTarget.learning_methods.value.trim(),
      meta_course_content: event.currentTarget.course_content.value.trim(),
    }));
  }

  render() {
    const {
      fetchingUniversities,
      fetchingUniversitiesError,
      universities,
      updateCourseFailed,
      course,
    } = this.props;

    return (
      <div>
        <Button onClick={() => this.handleOnClick()}>Edit course</Button>
        {this.state.showForm &&
          <Modal
            onDismiss={() => this.handleModalDismiss()}
            onSubmit={(event) => this.handleSubmit(event)}
            title="Update course"
          >
            { (fetchingUniversitiesError || updateCourseFailed) &&
              <Alert bsStyle="danger">
                <stong>Something went wrong, please try again later</stong>
              </Alert>
            }
            <Row>
              <Col xs={6} sm={6}>
                <FieldGroup
                  type="text"
                  label="Name (english)"
                  name="name"
                  placeholder="Enter text"
                  defaultValue={course.name}
                />
                <FieldGroup
                  type="text"
                  label="ID (e.g. tdt4120)"
                  name="course_id"
                  placeholder="Course id (e.g. tdt4120)"
                  defaultValue={course.course_id}
                />
                <FieldGroup
                  type="text"
                  label="Course level"
                  name="level"
                  placeholder="Course level"
                  defaultValue={course.meta_study_level_name}
                />
                <FormGroup>
                  <ControlLabel>University</ControlLabel>
                  { fetchingUniversities
                    ?
                      <Spinner />
                    :
                      <FormControl componentClass="select" name="university" defaultValue={course.university.id}>
                        {universities.map((university, key) => (
                          <option
                            value={university.id}
                            key={key}
                          >{university.name}
                          </option>
                        ))}
                      </FormControl>
                  }
                </FormGroup>
                <FormGroup>
                  <ControlLabel>Semester</ControlLabel>
                  <Radio
                    name="semester"
                    value="spring"
                    checked={this.state.semesterValue === 'spring'}
                    onChange={() => this.handleSemesterChange('spring')}
                  >
                    Spring
                  </Radio>
                  <Radio
                    name="semester"
                    value="autumn"
                    checked={this.state.semesterValue === 'autumn'}
                    onChange={() => this.handleSemesterChange('autumn')}
                  >
                    Autumn
                  </Radio>
                </FormGroup>
                <FieldGroup
                  type="text"
                  label="Credits"
                  name="credits"
                  placeholder="Number of credits"
                  defaultValue={course.credits}
                />
              </Col>
              <Col xs={6} sm={6}>
                <FormGroup controlId="formControlsTextarea">
                  <ControlLabel>Goals</ControlLabel>
                  <FormControl
                    componentClass="textarea"
                    placeholder="textarea"
                    defaultValue={course.meta_goals}
                    name="goals"
                  />
                </FormGroup>
                <FormGroup controlId="formControlsTextarea">
                  <ControlLabel>Learning methods</ControlLabel>
                  <FormControl
                    componentClass="textarea"
                    placeholder="textarea"
                    defaultValue={course.meta_learning_methods}
                    name="learning_methods"
                  />
                </FormGroup>
                <FormGroup controlId="formControlsTextarea">
                  <ControlLabel>Course content</ControlLabel>
                  <FormControl
                    componentClass="textarea"
                    placeholder="textarea"
                    defaultValue={course.meta_course_content}
                    name="course_content"
                  />
                </FormGroup>
              </Col>
            </Row>
          </Modal>
        }
      </div>
    );
  }
}

UpdateCourseButton.propTypes = {
  dispatch: PropTypes.func,
  universities: PropTypes.array,
  course: PropTypes.object,
  fetchingUniversities: PropTypes.bool,
  fetchingUniversitiesError: PropTypes.bool,
  requestingUpdateCourse: PropTypes.bool,
  updateCourseFailed: PropTypes.bool,
};

const mapStateToProps = createStructuredSelector({
  universities: selectUniversities(),
  fetchingUniversities: selectFetchingUniversities(),
  fetchingUniversitiesError: selectFetchingUniversitiesError(),
  requestingUpdateCourse: selectRequestingUpdateCourse(),
  updateCourseSuccess: selectUpdateCourseSuccess(),
  updateCourseFailed: selectUpdateCourseFailed(),
  course: selectCourse(),
});

export default connect(mapStateToProps)(UpdateCourseButton);
