import React, { PropTypes, Component } from 'react';
import { createStructuredSelector } from 'reselect';
import { connect } from 'react-redux';
import { push } from 'react-router-redux';

import Button from 'react-bootstrap/lib/Button';
import Radio from 'react-bootstrap/lib/Radio';
import FormGroup from 'react-bootstrap/lib/FormGroup';
import FormControl from 'react-bootstrap/lib/FormControl';
import ControlLabel from 'react-bootstrap/lib/ControlLabel';
import Alert from 'react-bootstrap/lib/Alert';

import Modal from 'components/Modal';
import FieldGroup from 'components/FieldGroup';
import Spinner from 'components/Spinner';

import {
  selectUniversities,
  selectFetchingUniversities,
  selectFetchingUniversitiesError,
  selectRequestingCreateCourse,
  selectCreateCourseSuccess,
  selectCreateCourseFailed,
  selectCourseId,
} from './selectors';
import { loadUniversityList, createCourse } from './actions';


class CreateCourseButton extends Component {
  constructor() {
    super();
    this.state = { showForm: false };
  }

  componentDidMount() {
    if (this.props.universities.length < 1) {
      this.props.dispatch(loadUniversityList());
    }
  }

  componentWillReceiveProps(next) {
    if (next.createCourseSuccess && this.state.showForm) {
      this.setState({ showForm: false });
      this.props.dispatch(push(`/course/${next.courseId}`));
    }
  }

  handleOnClick() {
    this.setState({ showForm: true });
  }

  handleModalDismiss() {
    this.setState({ showForm: false });
  }

  handleSubmit(event) {
    event.preventDefault();

    if (this.props.requestingCreateCourse) {
      return;
    }

    this.props.dispatch(createCourse({
      eng_name: event.currentTarget.name.value.trim(),
      course_id: event.currentTarget.course_id.value.trim(),
      level: event.currentTarget.level.value.trim(),
      university: event.currentTarget.university.value.trim(),
      semester: event.currentTarget.semester.value.trim(),
      credits: event.currentTarget.credits.value.trim(),
    }));
  }

  render() {
    const {
      fetchingUniversities,
      fetchingUniversitiesError,
      universities,
      createCourseFailed,
    } = this.props;
    return (
      <div>
        <Button onClick={() => this.handleOnClick()}>Add course</Button>
        {this.state.showForm &&
          <Modal
            onDismiss={() => this.handleModalDismiss()}
            onSubmit={(event) => this.handleSubmit(event)}
            title="Create course"
          >
            { (fetchingUniversitiesError || createCourseFailed) &&
              <Alert bsStyle="danger">
                <stong>Something went wrong, please try again later</stong>
              </Alert>
            }
            <FieldGroup
              type="text"
              label="Name (english)"
              name="name"
              placeholder="Enter text"
            />
            <FieldGroup
              type="text"
              label="ID (e.g. tdt4120)"
              name="course_id"
              placeholder="Course id (e.g. tdt4120)"
            />
            <FormGroup>
              <ControlLabel>Level</ControlLabel>
              <FormControl componentClass="select" name="level">
                <option value="" name="level">select</option>
                <option value="1">Introduction level</option>
                <option value="2">Something else level</option>
                <option value="3">Higher level</option>
              </FormControl>
            </FormGroup>
            <FormGroup>
              <ControlLabel>University</ControlLabel>
              { fetchingUniversities
                ?
                  <Spinner />
                :
                  <FormControl componentClass="select" name="university">
                    <option value="">select</option>
                    {universities.map((university, key) => (
                      <option value={university.id} key={key}>{university.name}</option>
                    ))}
                  </FormControl>
              }
            </FormGroup>
            <FormGroup>
              <ControlLabel>Semester</ControlLabel>
              <Radio checked readOnly name="semester" value="spring">Spring</Radio>
              <Radio readOnly name="semester" value="autumn">Autumn</Radio>
            </FormGroup>
            <FieldGroup
              type="text"
              label="Credits"
              name="credits"
              placeholder="Number of credits"
            />
          </Modal>
        }
      </div>
    );
  }
}

CreateCourseButton.propTypes = {
  dispatch: PropTypes.func,
  universities: PropTypes.array,
  fetchingUniversities: PropTypes.bool,
  fetchingUniversitiesError: PropTypes.bool,
  requestingCreateCourse: PropTypes.bool,
  createCourseFailed: PropTypes.bool,
};

const mapStateToProps = createStructuredSelector({
  universities: selectUniversities(),
  fetchingUniversities: selectFetchingUniversities(),
  fetchingUniversitiesError: selectFetchingUniversitiesError(),
  requestingCreateCourse: selectRequestingCreateCourse(),
  createCourseSuccess: selectCreateCourseSuccess(),
  createCourseFailed: selectCreateCourseFailed(),
  courseId: selectCourseId(),
});

export default connect(mapStateToProps)(CreateCourseButton);
